@@ -19,7 +19,15 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-from collections import MutableMapping, MutableSequence, OrderedDict
+# in Python 3.10, abstract collections have moved into their own module
+# for compatibility with 3.10+, first try imports from the new location
+# if that fails, try from the pre-3.10 location
+try:
+    from collections.abc import MutableMapping, MutableSequence
+    from collections import OrderedDict
+except:
+    from collections import MutableMapping, MutableSequence, OrderedDict
+    
 from datetime import datetime, timedelta, tzinfo
 from decimal import Decimal, ROUND_FLOOR, Context, Inexact
 from math import isnan
@@ -401,7 +409,7 @@ class Timestamp(datetime):
             * The ``precision`` field is passed as a keyword argument of the same name.
 
             * The ``fractional_precision`` field is passed as a keyword argument of the same name.
-              This field only relates to to the ``microseconds`` field and can be thought of
+              This field only relates to the ``microseconds`` field and can be thought of
               as the number of decimal digits that are significant.  This is an integer that
               that is in the closed interval ``[0, 6]``.  If ``0``, ``microseconds`` must be
               ``0`` indicating no precision below seconds.  This argument is optional and only valid
@@ -630,6 +638,16 @@ class Multimap(MutableMapping):
         for key in six.iterkeys(self.__store):
             yield key
 
+    def __str__(self):
+        return repr(self)
+
+    def __repr__(self):
+        str_repr = '{'
+        for key, value in self.items():
+            str_repr += '%r: %r, ' % (key, value)
+        str_repr = str_repr[:len(str_repr) - 2] + '}'
+        return six.ensure_binary(str_repr) if six.PY2 else str_repr
+
     def add_item(self, key, value):
         if key in self.__store:
             self.__store[key].append(value)
