@@ -23,6 +23,7 @@ from datetime import datetime
 from decimal import Decimal
 from io import BytesIO, TextIOBase
 from itertools import chain
+from types import GeneratorType
 
 import six
 
@@ -39,13 +40,20 @@ from .symbols import SymbolToken
 from .writer import blocking_writer
 from .writer_binary import binary_writer
 
+# Using C extension as default, and original python implementation if C extension doesn't exist.
+c_ext = True
+try:
+    import amazon.ion.ionc as ionc
+except ModuleNotFoundError:
+    c_ext = False
+
 
 _ION_CONTAINER_END_EVENT = IonEvent(IonEventType.CONTAINER_END)
 _IVM = b'\xe0\x01\x00\xea'
 _TEXT_TYPES = (TextIOBase, six.StringIO)
 
 
-def dump(obj, fp, imports=None, binary=True, sequence_as_stream=False, skipkeys=False, ensure_ascii=True,
+def dump_python(obj, fp, imports=None, binary=True, sequence_as_stream=False, skipkeys=False, ensure_ascii=True,
          check_circular=True, allow_nan=True, cls=None, indent=None, separators=None, encoding='utf-8', default=None,
          use_decimal=True, namedtuple_as_object=True, tuple_as_array=True, bigint_as_string=False, sort_keys=False,
          item_sort_key=None, for_json=None, ignore_nan=False, int_as_string_bitcount=None, iterable_as_array=False,
@@ -143,13 +151,12 @@ def dump(obj, fp, imports=None, binary=True, sequence_as_stream=False, skipkeys=
         **kw: NOT IMPLEMENTED
 
     """
-
     raw_writer = binary_writer(imports) if binary else text_writer(indent=indent)
     writer = blocking_writer(raw_writer, fp)
     from_type = _FROM_TYPE_TUPLE_AS_SEXP if tuple_as_sexp else _FROM_TYPE
     if binary or not omit_version_marker:
         writer.send(ION_VERSION_MARKER_EVENT)  # The IVM is emitted automatically in binary; it's optional in text.
-    if sequence_as_stream and isinstance(obj, (list, tuple)):
+    if sequence_as_stream and isinstance(obj, (list, tuple)) or isinstance(obj, GeneratorType):
         # Treat this top-level sequence as a stream; serialize its elements as top-level values, but don't serialize the
         # sequence itself.
         for top_level in obj:
@@ -209,6 +216,8 @@ def _dump(obj, writer, from_type, field=None, in_struct=False, depth=0):
     except AttributeError:
         ion_type = _ion_type(obj, from_type)
         ion_nature = False
+    if ion_type is None:
+        raise IonException('Value must have a non-None ion_type: %s, depth: %d, field: %s' % (repr(obj), depth, field))
     if not null and ion_type.is_container:
         if ion_nature:
             event = obj.to_event(IonEventType.CONTAINER_START, field_name=field, in_struct=in_struct, depth=depth)
@@ -295,8 +304,8 @@ def dumps(obj, imports=None, binary=True, sequence_as_stream=False, skipkeys=Fal
     return ret_val
 
 
-def load(fp, catalog=None, single_value=True, encoding='utf-8', cls=None, object_hook=None, parse_float=None,
-         parse_int=None, parse_constant=None, object_pairs_hook=None, use_decimal=None, **kw):
+def load_python(fp, catalog=None, single_value=True, encoding='utf-8', cls=None, object_hook=None, parse_float=None,
+        parse_int=None, parse_constant=None, object_pairs_hook=None, use_decimal=None, parse_eagerly=True, **kw):
     """Deserialize ``fp`` (a file-like object), which contains a text or binary Ion stream, to a Python object using the
     following conversion table::
         +-------------------+-------------------+
@@ -336,6 +345,8 @@ def load(fp, catalog=None, single_value=True, encoding='utf-8', cls=None, object
             will be returned without an enclosing container. If True and there are multiple top-level values in the Ion
             stream, IonException will be raised. NOTE: this means that when data is dumped using
             ``sequence_as_stream=True``, it must be loaded using ``single_value=False``. Default: True.
+        parse_eagerly: (Optional[True|False]) Used in conjunction with ``single_value=False`` to return the result as list
+            or an iterator
         encoding: NOT IMPLEMENTED
         cls: NOT IMPLEMENTED
         object_hook: NOT IMPLEMENTED
@@ -362,13 +373,24 @@ def load(fp, catalog=None, single_value=True, encoding='utf-8', cls=None, object
         else:
             raw_reader = text_reader()
     reader = blocking_reader(managed_reader(raw_reader, catalog), fp)
-    out = []  # top-level
-    _load(out, reader)
-    if single_value:
-        if len(out) != 1:
-            raise IonException('Stream contained %d values; expected a single value.' % (len(out),))
-        return out[0]
-    return out
+    if parse_eagerly:
+        out = []  # top-level
+        _load(out, reader)
+        if single_value:
+            if len(out) != 1:
+                raise IonException('Stream contained %d values; expected a single value.' % (len(out),))
+            return out[0]
+        return out
+    else:
+        out = _load_iteratively(reader)
+        if single_value:
+            result = next(out)
+            try:
+                next(out)
+                raise IonException('Stream contained more than 1 values; expected a single value.')
+            except StopIteration:
+                return result
+        return out
 
 
 _FROM_ION_TYPE = [
@@ -387,6 +409,21 @@ _FROM_ION_TYPE = [
     IonPyDict
 ]
 
+def _load_iteratively(reader, end_type=IonEventType.STREAM_END):
+    event = reader.send(NEXT_EVENT)
+    while event.event_type is not end_type:
+        ion_type = event.ion_type
+        if event.event_type is IonEventType.CONTAINER_START:
+            container = _FROM_ION_TYPE[ion_type].from_event(event)
+            _load(container, reader, IonEventType.CONTAINER_END, ion_type is IonType.STRUCT)
+            yield container
+        elif event.event_type is IonEventType.SCALAR:
+            if event.value is None or ion_type is IonType.NULL or ion_type.is_container:
+                scalar = IonPyNull.from_event(event)
+            else:
+                scalar = _FROM_ION_TYPE[ion_type].from_event(event)
+            yield scalar
+        event = reader.send(NEXT_EVENT)
 
 def _load(out, reader, end_type=IonEventType.STREAM_END, in_struct=False):
 
@@ -413,7 +450,7 @@ def _load(out, reader, end_type=IonEventType.STREAM_END, in_struct=False):
 
 
 def loads(ion_str, catalog=None, single_value=True, encoding='utf-8', cls=None, object_hook=None, parse_float=None,
-          parse_int=None, parse_constant=None, object_pairs_hook=None, use_decimal=None, **kw):
+          parse_int=None, parse_constant=None, object_pairs_hook=None, use_decimal=None, parse_eagerly=True, **kw):
     """Deserialize ``ion_str``, which is a string representation of an Ion object, to a Python object using the
     conversion table used by load (above).
 
@@ -424,6 +461,8 @@ def loads(ion_str, catalog=None, single_value=True, encoding='utf-8', cls=None,
             and will be returned without an enclosing container. If True and there are multiple top-level values in
             the Ion stream, IonException will be raised. NOTE: this means that when data is dumped using
             ``sequence_as_stream=True``, it must be loaded using ``single_value=False``. Default: True.
+        parse_eagerly: (Optional[True|False]) Used in conjunction with ``single_value=False`` to return the result as list
+            or an iterator
         encoding: NOT IMPLEMENTED
         cls: NOT IMPLEMENTED
         object_hook: NOT IMPLEMENTED
@@ -450,4 +489,61 @@ def loads(ion_str, catalog=None, single_value=True, encoding='utf-8', cls=None,
 
     return load(ion_buffer, catalog=catalog, single_value=single_value, encoding=encoding, cls=cls,
                 object_hook=object_hook, parse_float=parse_float, parse_int=parse_int, parse_constant=parse_constant,
-                object_pairs_hook=object_pairs_hook, use_decimal=use_decimal)
+                object_pairs_hook=object_pairs_hook, use_decimal=use_decimal, parse_eagerly=parse_eagerly)
+
+
+def dump_extension(obj, fp, binary=True, sequence_as_stream=False, tuple_as_sexp=False, omit_version_marker=False):
+    res = ionc.ionc_write(obj, binary, sequence_as_stream, tuple_as_sexp)
+
+    # TODO support "omit_version_marker" rather than hacking.
+    if not binary and not omit_version_marker:
+        res = b'$ion_1_0 ' + res
+    fp.write(res)
+
+
+def load_extension(fp, single_value=True, parse_eagerly=True):
+    iterator = ionc.ionc_read(fp, emit_bare_values=False)
+    if single_value:
+        try:
+            value = next(iterator)
+        except StopIteration:
+            return None
+        try:
+            next(iterator)
+            raise IonException('Stream contained more than 1 values; expected a single value.')
+        except StopIteration:
+            pass
+        return value
+    if parse_eagerly:
+        return list(iterator)
+    return iterator
+
+
+def dump(obj, fp, imports=None, binary=True, sequence_as_stream=False, skipkeys=False, ensure_ascii=True,
+         check_circular=True, allow_nan=True, cls=None, indent=None, separators=None, encoding='utf-8', default=None,
+         use_decimal=True, namedtuple_as_object=True, tuple_as_array=True, bigint_as_string=False, sort_keys=False,
+         item_sort_key=None, for_json=None, ignore_nan=False, int_as_string_bitcount=None, iterable_as_array=False,
+         tuple_as_sexp=False, omit_version_marker=False, **kw):
+    if c_ext and (imports is None and indent is None):
+        return dump_extension(obj, fp, binary=binary, sequence_as_stream=sequence_as_stream,
+                              tuple_as_sexp=tuple_as_sexp, omit_version_marker=omit_version_marker)
+    else:
+        return dump_python(obj, fp, imports=imports, binary=binary, sequence_as_stream=sequence_as_stream,
+                         skipkeys=skipkeys, ensure_ascii=ensure_ascii,check_circular=check_circular,
+                         allow_nan=allow_nan, cls=cls, indent=indent, separators=separators, encoding=encoding,
+                         default=default, use_decimal=use_decimal, namedtuple_as_object=namedtuple_as_object,
+                         tuple_as_array=tuple_as_array, bigint_as_string=bigint_as_string, sort_keys=sort_keys,
+                         item_sort_key=item_sort_key, for_json=for_json, ignore_nan=ignore_nan,
+                         int_as_string_bitcount=int_as_string_bitcount, iterable_as_array=iterable_as_array,
+                         tuple_as_sexp=tuple_as_sexp, omit_version_marker=omit_version_marker, **kw)
+
+
+def load(fp, catalog=None, single_value=True, encoding='utf-8', cls=None, object_hook=None, parse_float=None,
+         parse_int=None, parse_constant=None, object_pairs_hook=None, use_decimal=None, parse_eagerly=True, **kw):
+    if c_ext and catalog is None:
+        return load_extension(fp, parse_eagerly=parse_eagerly, single_value=single_value)
+    else:
+        return load_python(fp, catalog=catalog, single_value=single_value, encoding=encoding, cls=cls,
+                         object_hook=object_hook, parse_float=parse_float, parse_int=parse_int,
+                         parse_constant=parse_constant, object_pairs_hook=object_pairs_hook,
+                         use_decimal=use_decimal, parse_eagerly=parse_eagerly, **kw)
